#ifndef TABLES_H
#define TABLES_H

#include "pinball_entities.h"
#include "../Utilities/utilities.h"
#include "../Utilities/vector2D.h"
#include "stdint-gcc.h"
#include <num/num.h>
#include <vector>

extern Scene MyPinball;


libnum::num32 flipperHeight;
libnum::num32 cScale;
libnum::num32 simWidth;
libnum::num32 simHeight;

// extern bopti_image_t img_abyss;
extern uint8_t CurrentSequence;


void Setup_Table_0(void) {
  CurrentSequence = 0;

  libnum::num32 offset = libnum::num32(0.02);

  flipperHeight = libnum::num32(1.7);
  cScale = libnum::num32(azrp_height) / flipperHeight;
  simWidth = libnum::num32(azrp_width) / cScale;
  simHeight = libnum::num32(azrp_height) / cScale;

  MyPinball.has_locker = false;

  MyPinball.score = 0;
  MyPinball.gravity.Set(Vector2D(libnum::num32(0.0), libnum::num32(-1.0)));

  MyPinball.sideimage = nullptr;

  MyPinball.borders.clear();
  MyPinball.borders.emplace_back(libnum::num32(0.74), libnum::num32(0.25));
  MyPinball.borders.emplace_back(libnum::num32(1.0) - offset,
                                 libnum::num32(0.4));
  MyPinball.borders.emplace_back(libnum::num32(1.0) - offset,
                                 flipperHeight - offset);
  MyPinball.borders.emplace_back(offset, flipperHeight - offset);
  MyPinball.borders.emplace_back(offset, libnum::num32(0.4));
  MyPinball.borders.emplace_back(libnum::num32(0.26), libnum::num32(0.25));
  MyPinball.borders.emplace_back(libnum::num32(0.26), libnum::num32(offset));
  MyPinball.borders.emplace_back(libnum::num32(0.74), libnum::num32(offset));

  MyPinball.balls.clear();
  libnum::num32 ball_radius = libnum::num32(0.03);
  libnum::num32 ball_mass = libnum::num32(PI) * ball_radius * ball_radius;
  Vector2D ball_pos1(libnum::num32(0.92), libnum::num32(0.5));
  Vector2D ball_vel1(libnum::num32(0.2), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos1, ball_vel1,
                                 0.2, RGB565_BLOODYRED));
  Vector2D ball_pos2(libnum::num32(0.08), libnum::num32(0.5));
  Vector2D ball_vel2(libnum::num32(0.2), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos2, ball_vel2,
                                 0.2, RGB565_BLOODYRED));

  MyPinball.obstacles.clear();
  MyPinball.obstacles.emplace_back(
      0.10, Vector2D(libnum::num32(0.25), libnum::num32(0.6)), 2.0,
      RGB565_OCEANBLUE, 25);
  MyPinball.obstacles.emplace_back(
      0.10, Vector2D(libnum::num32(0.75), libnum::num32(0.5)), 2.0,
      RGB565_OCEANBLUE, 25);
  MyPinball.obstacles.emplace_back(
      0.12, Vector2D(libnum::num32(0.70), libnum::num32(1.0)), 2.0,
      RGB565_OCEANBLUE, 25);
  MyPinball.obstacles.emplace_back(
      0.10, Vector2D(libnum::num32(0.20), libnum::num32(1.2)), 2.0,
      RGB565_OCEANBLUE, 25);

  for (int i = 0; i < MyPinball.islands.size(); i++)
    MyPinball.islands[i].clear();
  MyPinball.islands.clear();

  MyPinball.flippers.clear();
  libnum::num32 flip_radius = libnum::num32(0.03);
  libnum::num32 flip_length = libnum::num32(0.2);
  libnum::num32 flip_maxRotation = libnum::num32(1.0);
  libnum::num32 flip_restAngle = libnum::num32(0.5);
  libnum::num32 flip_angularVelocity = libnum::num32(10.0);
  libnum::num32 flip_restitution = libnum::num32(0.0);

  Vector2D flip_pos1(libnum::num32(0.26), libnum::num32(0.22));
  MyPinball.flippers.emplace_back(
      flip_radius, flip_pos1, flip_length, -flip_restAngle, flip_maxRotation,
      flip_angularVelocity, flip_restitution, LEFT, RGB565_LEMONYELLOW);

  Vector2D flip_pos2(libnum::num32(0.74), libnum::num32(0.22));
  MyPinball.flippers.emplace_back(flip_radius, flip_pos2, flip_length,
                                  libnum::num32(PI) + flip_restAngle,
                                  -flip_maxRotation, flip_angularVelocity,
                                  flip_restitution, RIGHT, RGB565_LEMONYELLOW);

  //  Vector2D flip_pos3(libnum::num32(offset), libnum::num32(1.50));
  //  MyPinball.flippers.emplace_back( flip_radius, flip_pos3, flip_length,
  //  -flip_restAngle, flip_maxRotation, flip_angularVelocity, flip_restitution,
  //  LEFT, RGB565_LEMONYELLOW));

  //  Vector2D flip_pos4(libnum::num32(1.0)-offset, libnum::num32(1.25));
  //  MyPinball.flippers.emplace_back(  flip_radius, flip_pos4, flip_length,
  //  libnum::num32(PI) + flip_restAngle, -flip_maxRotation,
  //  flip_angularVelocity, flip_restitution, RIGHT, RGB565_LEMONYELLOW));
}

void Setup_Table_1(void) {
    CurrentSequence = 1;
  libnum::num32 offset = libnum::num32(0.02);

  flipperHeight = libnum::num32(1.7);
  cScale = libnum::num32(azrp_height) / flipperHeight;
  simWidth = libnum::num32(azrp_width) / cScale;
  simHeight = libnum::num32(azrp_height) / cScale;

  MyPinball.has_locker = false;

  MyPinball.score = 0;
  MyPinball.gravity.Set(Vector2D(libnum::num32(0.0), libnum::num32(-1.0)));

  MyPinball.sideimage = nullptr;

  MyPinball.borders.clear();
  MyPinball.borders.emplace_back(libnum::num32(0.74), libnum::num32(0.25));
  MyPinball.borders.emplace_back(libnum::num32(0.98), libnum::num32(0.40));
  MyPinball.borders.emplace_back(libnum::num32(0.98), libnum::num32(1.20));
  MyPinball.borders.emplace_back(libnum::num32(0.96), libnum::num32(1.34));
  MyPinball.borders.emplace_back(libnum::num32(0.90), libnum::num32(1.46));
  MyPinball.borders.emplace_back(libnum::num32(0.81), libnum::num32(1.56));
  MyPinball.borders.emplace_back(libnum::num32(0.70), libnum::num32(1.64));
  MyPinball.borders.emplace_back(libnum::num32(0.57), libnum::num32(1.68));
  MyPinball.borders.emplace_back(libnum::num32(0.43), libnum::num32(1.68));
  MyPinball.borders.emplace_back(libnum::num32(0.30), libnum::num32(1.64));
  MyPinball.borders.emplace_back(libnum::num32(0.19), libnum::num32(1.56));
  MyPinball.borders.emplace_back(libnum::num32(0.10), libnum::num32(1.46));
  MyPinball.borders.emplace_back(libnum::num32(0.04), libnum::num32(1.34));
  MyPinball.borders.emplace_back(libnum::num32(0.02), libnum::num32(1.20));
  MyPinball.borders.emplace_back(libnum::num32(0.02), libnum::num32(0.40));
  MyPinball.borders.emplace_back(libnum::num32(0.26), libnum::num32(0.25));
  MyPinball.borders.emplace_back(libnum::num32(0.26), libnum::num32(0.02));
  MyPinball.borders.emplace_back(libnum::num32(0.74), libnum::num32(0.02));

  MyPinball.balls.clear();
  libnum::num32 ball_radius = libnum::num32(0.03);
  libnum::num32 ball_mass = libnum::num32(PI) * ball_radius * ball_radius;
  Vector2D ball_pos1(libnum::num32(0.60), libnum::num32(0.5));
  Vector2D ball_vel1(libnum::num32(0.2), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos1, ball_vel1,
                                 0.2, RGB565_LEMONYELLOW));
  Vector2D ball_pos2(libnum::num32(0.40), libnum::num32(0.5));
  Vector2D ball_vel2(libnum::num32(-0.2), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos2, ball_vel2,
                                 0.2, RGB565_LEMONYELLOW));

  MyPinball.obstacles.clear();
  MyPinball.obstacles.emplace_back(
      0.08, Vector2D(libnum::num32(0.50), libnum::num32(1.45)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.08, Vector2D(libnum::num32(0.74), libnum::num32(1.20)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.08, Vector2D(libnum::num32(0.26), libnum::num32(1.20)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.08, Vector2D(libnum::num32(0.50), libnum::num32(0.95)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.13), libnum::num32(0.80)), 1.5,
      RGB565_LEMONYELLOW, 25);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.87), libnum::num32(0.80)), 1.5,
      RGB565_LEMONYELLOW, 25);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.15), libnum::num32(0.60)), 1.5,
      RGB565_GREEN, 10);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.85), libnum::num32(0.60)), 1.5,
      RGB565_GREEN, 10);

  for (int i = 0; i < MyPinball.islands.size(); i++)
    MyPinball.islands[i].clear();
  MyPinball.islands.clear();

  MyPinball.flippers.clear();
  libnum::num32 flip_radius = libnum::num32(0.03);
  libnum::num32 flip_length = libnum::num32(0.2);
  libnum::num32 flip_maxRotation = libnum::num32(1.0);
  libnum::num32 flip_restAngle = libnum::num32(0.5);
  libnum::num32 flip_angularVelocity = libnum::num32(10.0);
  libnum::num32 flip_restitution = libnum::num32(0.0);

  Vector2D flip_pos1(libnum::num32(0.26), libnum::num32(0.22));
  MyPinball.flippers.emplace_back(
      flip_radius, flip_pos1, flip_length, -flip_restAngle, flip_maxRotation,
      flip_angularVelocity, flip_restitution, LEFT, RGB565_OCEANBLUE);

  Vector2D flip_pos2(libnum::num32(0.74), libnum::num32(0.22));
  MyPinball.flippers.emplace_back(flip_radius, flip_pos2, flip_length,
                                  libnum::num32(PI) + flip_restAngle,
                                  -flip_maxRotation, flip_angularVelocity,
                                  flip_restitution, RIGHT, RGB565_OCEANBLUE);
}

void Setup_Table_2(void) {
  CurrentSequence = 2;
  libnum::num32 offset = libnum::num32(0.02);

  flipperHeight = libnum::num32(1.7);
  cScale = libnum::num32(azrp_height) / flipperHeight;
  simWidth = libnum::num32(azrp_width) / cScale;
  simHeight = libnum::num32(azrp_height) / cScale;

  MyPinball.has_locker = false;

  MyPinball.score = 0;
  MyPinball.gravity.Set(Vector2D(libnum::num32(0.0), libnum::num32(-1.0)));

  MyPinball.sideimage = nullptr;

  MyPinball.borders.clear();
  MyPinball.borders.emplace_back(libnum::num32(0.74), libnum::num32(0.25));
  MyPinball.borders.emplace_back(libnum::num32(0.98), libnum::num32(0.40));
  MyPinball.borders.emplace_back(libnum::num32(0.98), libnum::num32(1.20));
  MyPinball.borders.emplace_back(libnum::num32(0.96), libnum::num32(1.34));
  MyPinball.borders.emplace_back(libnum::num32(0.90), libnum::num32(1.46));
  MyPinball.borders.emplace_back(libnum::num32(0.81), libnum::num32(1.56));
  MyPinball.borders.emplace_back(libnum::num32(0.70), libnum::num32(1.64));
  MyPinball.borders.emplace_back(libnum::num32(0.57), libnum::num32(1.68));
  MyPinball.borders.emplace_back(libnum::num32(0.43), libnum::num32(1.68));
  MyPinball.borders.emplace_back(libnum::num32(0.30), libnum::num32(1.64));
  MyPinball.borders.emplace_back(libnum::num32(0.19), libnum::num32(1.56));
  MyPinball.borders.emplace_back(libnum::num32(0.10), libnum::num32(1.46));
  MyPinball.borders.emplace_back(libnum::num32(0.04), libnum::num32(1.34));
  MyPinball.borders.emplace_back(libnum::num32(0.02), libnum::num32(1.20));
  MyPinball.borders.emplace_back(libnum::num32(0.02), libnum::num32(0.40));
  MyPinball.borders.emplace_back(libnum::num32(0.26), libnum::num32(0.25));
  MyPinball.borders.emplace_back(libnum::num32(0.26), libnum::num32(0.02));
  MyPinball.borders.emplace_back(libnum::num32(0.74), libnum::num32(0.02));

  MyPinball.balls.clear();
  libnum::num32 ball_radius = libnum::num32(0.03);
  libnum::num32 ball_mass = libnum::num32(PI) * ball_radius * ball_radius;
  Vector2D ball_pos1(libnum::num32(0.70), libnum::num32(0.5));
  Vector2D ball_vel1(libnum::num32(0.2), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos1, ball_vel1,
                                 0.2, RGB565_LEMONYELLOW));
  Vector2D ball_pos2(libnum::num32(0.30), libnum::num32(0.5));
  Vector2D ball_vel2(libnum::num32(-0.2), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos2, ball_vel2,
                                 0.2, RGB565_LEMONYELLOW));

  MyPinball.obstacles.clear();
  MyPinball.obstacles.emplace_back(
      0.06, Vector2D(libnum::num32(0.50), libnum::num32(1.45)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.06, Vector2D(libnum::num32(0.74), libnum::num32(1.20)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.06, Vector2D(libnum::num32(0.26), libnum::num32(1.20)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.06, Vector2D(libnum::num32(0.50), libnum::num32(0.95)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.13), libnum::num32(0.80)), 1.5,
      RGB565_LEMONYELLOW, 25);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.87), libnum::num32(0.80)), 1.5,
      RGB565_LEMONYELLOW, 25);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.15), libnum::num32(0.60)), 1.5,
      RGB565_GREEN, 10);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.85), libnum::num32(0.60)), 1.5,
      RGB565_GREEN, 10);

  for (int i = 0; i < MyPinball.islands.size(); i++)
    MyPinball.islands[i].clear();
  MyPinball.islands.clear();

  std::vector<Vector2D> island1;
  island1.emplace_back(libnum::num32(0.50), libnum::num32(1.10));
  island1.emplace_back(libnum::num32(0.40), libnum::num32(1.20));
  island1.emplace_back(libnum::num32(0.50), libnum::num32(1.30));
  island1.emplace_back(libnum::num32(0.60), libnum::num32(1.20));
  MyPinball.islands.push_back(island1);

  std::vector<Vector2D> island2;
  island2.emplace_back(libnum::num32(0.50), libnum::num32(0.50));
  island2.emplace_back(libnum::num32(0.40), libnum::num32(0.70));
  island2.emplace_back(libnum::num32(0.50), libnum::num32(0.72));
  island2.emplace_back(libnum::num32(0.60), libnum::num32(0.70));
  MyPinball.islands.push_back(island2);

  MyPinball.flippers.clear();
  libnum::num32 flip_radius = libnum::num32(0.03);
  libnum::num32 flip_length = libnum::num32(0.2);
  libnum::num32 flip_maxRotation = libnum::num32(1.0);
  libnum::num32 flip_restAngle = libnum::num32(0.5);
  libnum::num32 flip_angularVelocity = libnum::num32(10.0);
  libnum::num32 flip_restitution = libnum::num32(0.0);

  Vector2D flip_pos1(libnum::num32(0.26), libnum::num32(0.22));
  MyPinball.flippers.emplace_back(
      flip_radius, flip_pos1, flip_length, -flip_restAngle, flip_maxRotation,
      flip_angularVelocity, flip_restitution, LEFT, RGB565_OCEANBLUE);

  Vector2D flip_pos2(libnum::num32(0.74), libnum::num32(0.22));
  MyPinball.flippers.emplace_back(flip_radius, flip_pos2, flip_length,
                                  libnum::num32(PI) + flip_restAngle,
                                  -flip_maxRotation, flip_angularVelocity,
                                  flip_restitution, RIGHT, RGB565_OCEANBLUE);
}

void Setup_Table_3(void) {
    CurrentSequence = 3;
  libnum::num32 offset = libnum::num32(0.02);

  flipperHeight = libnum::num32(1.7);
  cScale = libnum::num32(azrp_height) / flipperHeight;
  simWidth = libnum::num32(azrp_width) / cScale;
  simHeight = libnum::num32(azrp_height) / cScale;

  MyPinball.has_locker = false;

  MyPinball.score = 0;
  MyPinball.gravity.Set(Vector2D(libnum::num32(0.0), libnum::num32(-1.0)));

  MyPinball.sideimage = nullptr; // &img_abyss;

  MyPinball.borders.clear();
  MyPinball.borders.emplace_back(libnum::num32(0.74), libnum::num32(0.25));
  MyPinball.borders.emplace_back(libnum::num32(0.98), libnum::num32(0.40));
  MyPinball.borders.emplace_back(libnum::num32(0.98), libnum::num32(1.20));
  MyPinball.borders.emplace_back(libnum::num32(0.96), libnum::num32(1.34));
  MyPinball.borders.emplace_back(libnum::num32(0.90), libnum::num32(1.46));
  MyPinball.borders.emplace_back(libnum::num32(0.81), libnum::num32(1.56));
  MyPinball.borders.emplace_back(libnum::num32(0.70), libnum::num32(1.64));
  MyPinball.borders.emplace_back(libnum::num32(0.57), libnum::num32(1.68));
  MyPinball.borders.emplace_back(libnum::num32(0.43), libnum::num32(1.68));
  MyPinball.borders.emplace_back(libnum::num32(0.30), libnum::num32(1.64));
  MyPinball.borders.emplace_back(libnum::num32(0.19), libnum::num32(1.56));
  MyPinball.borders.emplace_back(libnum::num32(0.10), libnum::num32(1.46));
  MyPinball.borders.emplace_back(libnum::num32(0.04), libnum::num32(1.34));
  MyPinball.borders.emplace_back(libnum::num32(0.02), libnum::num32(1.20));
  MyPinball.borders.emplace_back(libnum::num32(0.02), libnum::num32(0.40));
  MyPinball.borders.emplace_back(libnum::num32(0.26), libnum::num32(0.25));
  MyPinball.borders.emplace_back(libnum::num32(0.26), libnum::num32(0.02));
  MyPinball.borders.emplace_back(libnum::num32(0.74), libnum::num32(0.02));

  MyPinball.balls.clear();
  libnum::num32 ball_radius = libnum::num32(0.03);
  libnum::num32 ball_mass = libnum::num32(PI) * ball_radius * ball_radius;
  Vector2D ball_pos1(libnum::num32(0.70), libnum::num32(0.70));
  Vector2D ball_vel1(libnum::num32(0.2), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos1, ball_vel1,
                                 0.2, RGB565_LEMONYELLOW));
  Vector2D ball_pos2(libnum::num32(0.30), libnum::num32(0.70));
  Vector2D ball_vel2(libnum::num32(-0.2), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos2, ball_vel2,
                                 0.2, RGB565_LEMONYELLOW));

  MyPinball.obstacles.clear();
  MyPinball.obstacles.emplace_back(
      0.06, Vector2D(libnum::num32(0.50), libnum::num32(1.45)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.06, Vector2D(libnum::num32(0.74), libnum::num32(1.20)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.06, Vector2D(libnum::num32(0.26), libnum::num32(1.20)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.06, Vector2D(libnum::num32(0.50), libnum::num32(0.95)), 1.5, RGB565_RED,
      100);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.15), libnum::num32(0.80)), 1.5,
      RGB565_LEMONYELLOW, 25);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.85), libnum::num32(0.80)), 1.5,
      RGB565_LEMONYELLOW, 25);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.30), libnum::num32(0.60)), 1.5,
      RGB565_GREEN, 10);
  MyPinball.obstacles.emplace_back(
      0.04, Vector2D(libnum::num32(0.70), libnum::num32(0.60)), 1.5,
      RGB565_GREEN, 10);

  for (int i = 0; i < MyPinball.islands.size(); i++)
    MyPinball.islands[i].clear();
  MyPinball.islands.clear();

  std::vector<Vector2D> island1;
    island1.emplace_back( libnum::num32(0.50), libnum::num32(1.10) );
    island1.emplace_back( libnum::num32(0.40), libnum::num32(1.20) );
    island1.emplace_back( libnum::num32(0.50), libnum::num32(1.30) );
    island1.emplace_back( libnum::num32(0.60), libnum::num32(1.20) );
    MyPinball.islands.push_back(island1);

    std::vector<Vector2D> island2;
    island2.emplace_back( libnum::num32(0.50), libnum::num32(0.60) );
    island2.emplace_back( libnum::num32(0.40), libnum::num32(0.80) );
    island2.emplace_back( libnum::num32(0.50), libnum::num32(0.82) );
    island2.emplace_back( libnum::num32(0.60), libnum::num32(0.80) );
    MyPinball.islands.push_back(island2);

    std::vector<Vector2D> island3;
    island3.emplace_back( libnum::num32(0.12), libnum::num32(0.45) );
    island3.emplace_back( libnum::num32(0.12), libnum::num32(0.65) );
    island3.emplace_back( libnum::num32(0.26), libnum::num32(0.37) );
    MyPinball.islands.push_back(island3);

    std::vector<Vector2D> island4;
    island4.emplace_back( libnum::num32(0.88), libnum::num32(0.65) );
    island4.emplace_back( libnum::num32(0.88), libnum::num32(0.45) );
    island4.emplace_back( libnum::num32(0.74), libnum::num32(0.37) );
    MyPinball.islands.push_back(island4);

    MyPinball.flippers.clear();
    libnum::num32 flip_radius = libnum::num32(0.03);
    libnum::num32 flip_length = libnum::num32(0.2);
    libnum::num32 flip_maxRotation = libnum::num32(1.0);
    libnum::num32 flip_restAngle = libnum::num32(0.5);
    libnum::num32 flip_angularVelocity = libnum::num32(10.0);
    libnum::num32 flip_restitution = libnum::num32(0.0);

    Vector2D flip_pos1(libnum::num32(0.26), libnum::num32(0.22));
    MyPinball.flippers.emplace_back(
        flip_radius, flip_pos1, flip_length, -flip_restAngle, flip_maxRotation,
        flip_angularVelocity, flip_restitution, LEFT, RGB565_OCEANBLUE);

    Vector2D flip_pos2(libnum::num32(0.74), libnum::num32(0.22));
    MyPinball.flippers.emplace_back(flip_radius, flip_pos2, flip_length,
                                    libnum::num32(PI) + flip_restAngle,
                                    -flip_maxRotation, flip_angularVelocity,
                                    flip_restitution, RIGHT, RGB565_OCEANBLUE);
}

void Setup_Table_4(void) {
    CurrentSequence = 4;
  libnum::num32 offset = libnum::num32(0.02);

  flipperHeight = libnum::num32(1.7);
  cScale = libnum::num32(azrp_height) / flipperHeight;
  simWidth = libnum::num32(azrp_width) / cScale;
  simHeight = libnum::num32(azrp_height) / cScale;

  MyPinball.has_locker = true;

  MyPinball.score = 0;
  MyPinball.gravity.Set(Vector2D(libnum::num32(0.0), libnum::num32(-1.0)));

  MyPinball.sideimage = nullptr ; //&img_abyss;

  MyPinball.borders.clear();
  MyPinball.borders.emplace_back( libnum::num32(0.74), libnum::num32(0.25) );
  MyPinball.borders.emplace_back( libnum::num32(0.98), libnum::num32(0.40) );
  MyPinball.borders.emplace_back( libnum::num32(0.98), libnum::num32(1.10) );

  MyPinball.borders.emplace_back( libnum::num32(1.04), libnum::num32(1.10) );
  MyPinball.borders.emplace_back( libnum::num32(1.04), libnum::num32(0.10) );
  MyPinball.borders.emplace_back( libnum::num32(1.11), libnum::num32(0.10) );
  MyPinball.borders.emplace_back( libnum::num32(1.11), libnum::num32(1.20) );

  MyPinball.borders.emplace_back( libnum::num32(0.90), libnum::num32(1.46) );
  MyPinball.borders.emplace_back( libnum::num32(0.81), libnum::num32(1.56) );
  MyPinball.borders.emplace_back( libnum::num32(0.70), libnum::num32(1.64) );
  MyPinball.borders.emplace_back( libnum::num32(0.57), libnum::num32(1.68) );
  MyPinball.borders.emplace_back( libnum::num32(0.43), libnum::num32(1.68) );
  MyPinball.borders.emplace_back( libnum::num32(0.30), libnum::num32(1.64) );
  MyPinball.borders.emplace_back( libnum::num32(0.19), libnum::num32(1.56) );
  MyPinball.borders.emplace_back( libnum::num32(0.10), libnum::num32(1.46) );
  MyPinball.borders.emplace_back( libnum::num32(0.04), libnum::num32(1.34) );
  MyPinball.borders.emplace_back( libnum::num32(0.02), libnum::num32(1.20) );
  MyPinball.borders.emplace_back( libnum::num32(0.02), libnum::num32(0.40) );
  MyPinball.borders.emplace_back( libnum::num32(0.26), libnum::num32(0.25) );
  MyPinball.borders.emplace_back( libnum::num32(0.26), libnum::num32(0.02) );
  MyPinball.borders.emplace_back( libnum::num32(0.74), libnum::num32(0.02) );


  MyPinball.has_locker = true;
  MyPinball.locker.clear();
  MyPinball.locker.emplace_back( libnum::num32(0.90), libnum::num32(1.46) );
  MyPinball.locker.emplace_back( libnum::num32(1.11), libnum::num32(1.20) );
  MyPinball.locker.emplace_back( libnum::num32(1.11), libnum::num32(1.10) );
  MyPinball.locker.emplace_back( libnum::num32(1.04), libnum::num32(1.10) );
  MyPinball.locker.emplace_back( libnum::num32(0.98), libnum::num32(1.10) );
  MyPinball.locker.emplace_back( libnum::num32(0.98), libnum::num32(1.20) );
  MyPinball.locker.emplace_back( libnum::num32(0.96), libnum::num32(1.34) );
  MyPinball.locker_is_enabled = false;


  MyPinball.Lock_radius = libnum::num(0.03);
  MyPinball.Lock_pos = Vector2D(libnum::num32(0.50), libnum::num32(1.64));
  MyPinball.Lock_color = RGB565_LEMONYELLOW;



  MyPinball.balls.clear();
  libnum::num32 ball_radius = libnum::num32(0.03);
  libnum::num32 ball_mass = libnum::num32(PI) * ball_radius * ball_radius;

  Vector2D ball_pos1(libnum::num32(1.075), libnum::num32(0.15));
  Vector2D ball_vel1(libnum::num32(0.0), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos1, ball_vel1,
                                 0.2, RGB565_LEMONYELLOW));

/*
  Vector2D ball_pos2(libnum::num32(1.075), libnum::num32(0.45));
  Vector2D ball_vel2(libnum::num32(0.0), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos2, ball_vel2,
                                 0.2, RGB565_LEMONYELLOW));
*/

  MyPinball.obstacles.clear();
  MyPinball.obstacles.reserve(16);

  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.50), libnum::num32(1.45)), 1.5, RGB565_RED, 100 );
  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.74), libnum::num32(1.20)), 1.5, RGB565_RED, 100 );
  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.26), libnum::num32(1.20)), 1.5, RGB565_RED, 100 );
  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.50), libnum::num32(0.95)), 1.5, RGB565_RED, 100 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(0.15), libnum::num32(0.80)), 1.5, RGB565_LEMONYELLOW, 25 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(0.85), libnum::num32(0.80)), 1.5, RGB565_LEMONYELLOW, 25 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(0.30), libnum::num32(0.60)), 1.5, RGB565_GREEN, 10 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(0.70), libnum::num32(0.60)), 1.5, RGB565_GREEN, 10 );

  for (int i = 0; i < MyPinball.islands.size(); i++)
    MyPinball.islands[i].clear();
  MyPinball.islands.clear();

  std::vector<Vector2D> island1;
    island1.emplace_back( libnum::num32(0.50), libnum::num32(1.15) );
    island1.emplace_back( libnum::num32(0.45), libnum::num32(1.20) );
    island1.emplace_back( libnum::num32(0.50), libnum::num32(1.25) );
    island1.emplace_back( libnum::num32(0.55), libnum::num32(1.20) );
    MyPinball.islands.push_back(island1);

    std::vector<Vector2D> island2;
    island2.emplace_back( libnum::num32(0.50), libnum::num32(0.60) );
    island2.emplace_back( libnum::num32(0.40), libnum::num32(0.80) );
    island2.emplace_back( libnum::num32(0.50), libnum::num32(0.82) );
    island2.emplace_back( libnum::num32(0.60), libnum::num32(0.80) );
    MyPinball.islands.push_back(island2);

    std::vector<Vector2D> island3;
    island3.emplace_back( libnum::num32(0.12), libnum::num32(0.45) );
    island3.emplace_back( libnum::num32(0.12), libnum::num32(0.65) );
    island3.emplace_back( libnum::num32(0.26), libnum::num32(0.37) );
    MyPinball.islands.push_back(island3);

    std::vector<Vector2D> island4;
    island4.emplace_back( libnum::num32(0.88), libnum::num32(0.65) );
    island4.emplace_back( libnum::num32(0.88), libnum::num32(0.45) );
    island4.emplace_back( libnum::num32(0.74), libnum::num32(0.37) );
    MyPinball.islands.push_back(island4);

    MyPinball.flippers.clear();
    libnum::num32 flip_radius = libnum::num32(0.03);
    libnum::num32 flip_length = libnum::num32(0.2);
    libnum::num32 flip_maxRotation = libnum::num32(1.0);
    libnum::num32 flip_restAngle = libnum::num32(0.5);
    libnum::num32 flip_angularVelocity = libnum::num32(10.0);
    libnum::num32 flip_restitution = libnum::num32(0.0);

    Vector2D flip_pos1(libnum::num32(0.26), libnum::num32(0.22));
    MyPinball.flippers.emplace_back( flip_radius, flip_pos1, flip_length, -flip_restAngle, flip_maxRotation, flip_angularVelocity, flip_restitution, LEFT, RGB565_OCEANBLUE );

    Vector2D flip_pos2(libnum::num32(0.74), libnum::num32(0.22));
    MyPinball.flippers.emplace_back(  flip_radius, flip_pos2, flip_length, libnum::num32(PI) + flip_restAngle, -flip_maxRotation, flip_angularVelocity, flip_restitution, RIGHT, RGB565_OCEANBLUE );
}


void Setup_Table_5(void) {
  CurrentSequence = 5;
  libnum::num32 offset = libnum::num32(0.02);

  flipperHeight = libnum::num32(1.7);
  cScale = libnum::num32(azrp_height) / flipperHeight;
  simWidth = libnum::num32(azrp_width) / cScale;
  simHeight = libnum::num32(azrp_height) / cScale;

  MyPinball.has_locker = true;

  MyPinball.score = 0;
  MyPinball.gravity.Set(Vector2D(libnum::num32(0.0), libnum::num32(-0.5)));

  MyPinball.sideimage = nullptr;

  MyPinball.borders.clear();
  MyPinball.borders.emplace_back( libnum::num32(0.74), libnum::num32(0.25) );
  MyPinball.borders.emplace_back( libnum::num32(0.98), libnum::num32(0.40) );
  MyPinball.borders.emplace_back( libnum::num32(0.98), libnum::num32(1.10) );

  MyPinball.borders.emplace_back( libnum::num32(1.04), libnum::num32(1.10) );
  MyPinball.borders.emplace_back( libnum::num32(1.04), libnum::num32(0.10) );
  MyPinball.borders.emplace_back( libnum::num32(1.11), libnum::num32(0.10) );
  MyPinball.borders.emplace_back( libnum::num32(1.11), libnum::num32(1.20) );

  MyPinball.borders.emplace_back( libnum::num32(0.90), libnum::num32(1.46) );
  MyPinball.borders.emplace_back( libnum::num32(0.81), libnum::num32(1.56) );
  MyPinball.borders.emplace_back( libnum::num32(0.70), libnum::num32(1.64) );
  MyPinball.borders.emplace_back( libnum::num32(0.57), libnum::num32(1.68) );

  MyPinball.borders.emplace_back( libnum::num32(0.50), libnum::num32(1.64) );
  
  MyPinball.borders.emplace_back( libnum::num32(0.43), libnum::num32(1.68) );
  MyPinball.borders.emplace_back( libnum::num32(0.30), libnum::num32(1.64) );
  MyPinball.borders.emplace_back( libnum::num32(0.19), libnum::num32(1.56) );
  MyPinball.borders.emplace_back( libnum::num32(0.10), libnum::num32(1.46) );
  MyPinball.borders.emplace_back( libnum::num32(0.04), libnum::num32(1.34) );
  MyPinball.borders.emplace_back( libnum::num32(0.02), libnum::num32(1.20) );
  MyPinball.borders.emplace_back( libnum::num32(0.02), libnum::num32(0.40) );
  MyPinball.borders.emplace_back( libnum::num32(0.26), libnum::num32(0.25) );
  MyPinball.borders.emplace_back( libnum::num32(0.26), libnum::num32(0.15) );
  MyPinball.borders.emplace_back( libnum::num32(0.42), libnum::num32(0.02) );
  MyPinball.borders.emplace_back( libnum::num32(0.58), libnum::num32(0.02) );
  MyPinball.borders.emplace_back( libnum::num32(0.74), libnum::num32(0.15) );


  MyPinball.has_locker = true;
  MyPinball.locker.clear();
  MyPinball.locker.emplace_back( libnum::num32(0.90), libnum::num32(1.46) );
  MyPinball.locker.emplace_back( libnum::num32(1.11), libnum::num32(1.20) );
  MyPinball.locker.emplace_back( libnum::num32(1.11), libnum::num32(1.10) );
  MyPinball.locker.emplace_back( libnum::num32(1.04), libnum::num32(1.10) );
  MyPinball.locker.emplace_back( libnum::num32(0.98), libnum::num32(1.10) );
  MyPinball.locker.emplace_back( libnum::num32(0.98), libnum::num32(1.20) );
  MyPinball.locker.emplace_back( libnum::num32(0.96), libnum::num32(1.34) );
  MyPinball.locker_is_enabled = false;


  MyPinball.Lock_radius = libnum::num(0.03);
  MyPinball.Lock_pos = Vector2D(libnum::num32(0.50), libnum::num32(1.64));
  MyPinball.Lock_color = RGB565_LEMONYELLOW;


  MyPinball.BucketMin = Vector2D( libnum::num32(0.42), libnum::num32(0.02) );
  MyPinball.BucketMax = Vector2D( libnum::num32(0.58), libnum::num32(0.10) );
  

  MyPinball.balls.clear();
  libnum::num32 ball_radius = libnum::num32(0.03);
  libnum::num32 ball_mass = libnum::num32(PI) * ball_radius * ball_radius;

  Vector2D ball_pos1(libnum::num32(1.075), libnum::num32(0.15));
//  Vector2D ball_vel1(libnum::num32(0.0), libnum::num32(3.5));
  Vector2D ball_vel1(libnum::num32(0.0), libnum::num32(0.0));

  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos1, ball_vel1,
                                 0.2, RGB565_LEMONYELLOW));

  MyPinball.CurrentBallToLaunch = 0;
  MyPinball.InitialPosition = ball_pos1.Clone();

/*
  Vector2D ball_pos2(libnum::num32(1.075), libnum::num32(0.45));
  Vector2D ball_vel2(libnum::num32(0.0), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos2, ball_vel2,
                                 0.2, RGB565_LEMONYELLOW));
*/

  MyPinball.obstacles.clear();
  MyPinball.obstacles.reserve(16);

  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.50), libnum::num32(1.45)), 1.5, RGB565_RED, 100 );
  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.74), libnum::num32(1.20)), 1.5, RGB565_RED, 100 );
  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.26), libnum::num32(1.20)), 1.5, RGB565_RED, 100 );
  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.50), libnum::num32(0.95)), 1.5, RGB565_RED, 100 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(0.15), libnum::num32(0.80)), 1.5, RGB565_LEMONYELLOW, 25 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(0.85), libnum::num32(0.80)), 1.5, RGB565_LEMONYELLOW, 25 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(0.30), libnum::num32(0.60)), 1.5, RGB565_GREEN, 10 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(0.70), libnum::num32(0.60)), 1.5, RGB565_GREEN, 10 );

  for (int i = 0; i < MyPinball.islands.size(); i++)
    MyPinball.islands[i].clear();
  MyPinball.islands.clear();

  std::vector<Vector2D> island1;
    island1.emplace_back( libnum::num32(0.50), libnum::num32(1.13) );
    island1.emplace_back( libnum::num32(0.43), libnum::num32(1.20) );
    island1.emplace_back( libnum::num32(0.50), libnum::num32(1.27) );
    island1.emplace_back( libnum::num32(0.57), libnum::num32(1.20) );
    MyPinball.islands.push_back(island1);

    std::vector<Vector2D> island2;
    island2.emplace_back( libnum::num32(0.49), libnum::num32(0.61) );
    island2.emplace_back( libnum::num32(0.40), libnum::num32(0.80) );
    island2.emplace_back( libnum::num32(0.50), libnum::num32(0.82) );
    island2.emplace_back( libnum::num32(0.60), libnum::num32(0.80) );
    island2.emplace_back( libnum::num32(0.51), libnum::num32(0.61) );
    MyPinball.islands.push_back(island2);

    std::vector<Vector2D> island3;
    island3.emplace_back( libnum::num32(0.12), libnum::num32(0.45) );
    island3.emplace_back( libnum::num32(0.12), libnum::num32(0.65) );
    island3.emplace_back( libnum::num32(0.14), libnum::num32(0.65) );
    island3.emplace_back( libnum::num32(0.26), libnum::num32(0.39) );
    island3.emplace_back( libnum::num32(0.26), libnum::num32(0.37) );
    MyPinball.islands.push_back(island3);

    std::vector<Vector2D> island4;
    island4.emplace_back( libnum::num32(0.88), libnum::num32(0.65) );
    island4.emplace_back( libnum::num32(0.88), libnum::num32(0.45) );
    island4.emplace_back( libnum::num32(0.74), libnum::num32(0.37) );
    island4.emplace_back( libnum::num32(0.74), libnum::num32(0.39) );
    island4.emplace_back( libnum::num32(0.86), libnum::num32(0.65) );
    MyPinball.islands.push_back(island4);

    MyPinball.flippers.clear();
    libnum::num32 flip_radius = libnum::num32(0.03);
    libnum::num32 flip_length = libnum::num32(0.2);
    libnum::num32 flip_length_short = libnum::num32(0.12);
    libnum::num32 flip_maxRotation = libnum::num32(1.0);
    libnum::num32 flip_restAngle = libnum::num32(0.5);
    libnum::num32 flip_angularVelocity = libnum::num32(10.0);
    libnum::num32 flip_restitution = libnum::num32(0.0);

    Vector2D flip_pos1(libnum::num32(0.26), libnum::num32(0.22));
    MyPinball.flippers.emplace_back( flip_radius, flip_pos1, flip_length, -flip_restAngle, flip_maxRotation, flip_angularVelocity, flip_restitution, LEFT, RGB565_OCEANBLUE );

    Vector2D flip_pos2(libnum::num32(0.74), libnum::num32(0.22));
    MyPinball.flippers.emplace_back(  flip_radius, flip_pos2, flip_length, libnum::num32(PI) + flip_restAngle, -flip_maxRotation, flip_angularVelocity, flip_restitution, RIGHT, RGB565_OCEANBLUE );

/*
    Vector2D flip_pos3(libnum::num32(0.02), libnum::num32(1));
    MyPinball.flippers.emplace_back( flip_radius, flip_pos3, flip_length_short, -flip_restAngle, flip_maxRotation, flip_angularVelocity, flip_restitution, LEFT, RGB565_OCEANBLUE );

    Vector2D flip_pos4(libnum::num32(0.98), libnum::num32(1));
    MyPinball.flippers.emplace_back(  flip_radius, flip_pos4, flip_length_short, libnum::num32(PI) + flip_restAngle, -flip_maxRotation, flip_angularVelocity, flip_restitution, RIGHT, RGB565_OCEANBLUE );
*/
}



void Setup_Table_6(void) {
  CurrentSequence = 6;
  libnum::num32 offset = libnum::num32(0.02);

  flipperHeight = libnum::num32(1.9);
  cScale = libnum::num32(azrp_height) / flipperHeight;
  simWidth = libnum::num32(azrp_width) / cScale;
  simHeight = libnum::num32(azrp_height) / cScale;

  MyPinball.has_locker = true;

  MyPinball.score = 0;
  MyPinball.gravity.Set(Vector2D(libnum::num32(0.0), libnum::num32(-0.5)));

  MyPinball.sideimage = nullptr;

  MyPinball.borders.clear();
  MyPinball.borders.emplace_back( libnum::num32(1.74), libnum::num32(0.25) );
  MyPinball.borders.emplace_back( libnum::num32(1.98), libnum::num32(0.40) );
  MyPinball.borders.emplace_back( libnum::num32(1.98), libnum::num32(1.10) );
  // launcher
  MyPinball.borders.emplace_back( libnum::num32(2.02), libnum::num32(1.10) );
  MyPinball.borders.emplace_back( libnum::num32(2.02), libnum::num32(0.80) );
  MyPinball.borders.emplace_back( libnum::num32(2.09), libnum::num32(0.80) );
  MyPinball.borders.emplace_back( libnum::num32(2.09), libnum::num32(1.20) );
  MyPinball.borders.emplace_back( libnum::num32(1.90), libnum::num32(1.46) );
  // top circular part
  MyPinball.borders.emplace_back( libnum::num32(1.81), libnum::num32(1.56) );
  MyPinball.borders.emplace_back( libnum::num32(1.70), libnum::num32(1.64) );
  MyPinball.borders.emplace_back( libnum::num32(1.57), libnum::num32(1.68) );
  //MyPinball.borders.emplace_back( libnum::num32(1.50), libnum::num32(1.64) );
  MyPinball.borders.emplace_back( libnum::num32(1.43), libnum::num32(1.68) );
  MyPinball.borders.emplace_back( libnum::num32(1.30), libnum::num32(1.64) );
  MyPinball.borders.emplace_back( libnum::num32(1.19), libnum::num32(1.56) );
  MyPinball.borders.emplace_back( libnum::num32(1.10), libnum::num32(1.46) );
  MyPinball.borders.emplace_back( libnum::num32(1.08), libnum::num32(1.40) );

  // bridge between the two parts of the table
  MyPinball.borders.emplace_back(libnum::num32(0.96), libnum::num32(1.44));
  MyPinball.borders.emplace_back(libnum::num32(0.90), libnum::num32(1.56));
  MyPinball.borders.emplace_back(libnum::num32(0.81), libnum::num32(1.66));
  MyPinball.borders.emplace_back(libnum::num32(0.70), libnum::num32(1.74));
  MyPinball.borders.emplace_back(libnum::num32(0.57), libnum::num32(1.78));
  MyPinball.borders.emplace_back(libnum::num32(0.43), libnum::num32(1.78));
  MyPinball.borders.emplace_back(libnum::num32(0.30), libnum::num32(1.74));
  MyPinball.borders.emplace_back(libnum::num32(0.19), libnum::num32(1.66));
  MyPinball.borders.emplace_back(libnum::num32(0.10), libnum::num32(1.56));
  MyPinball.borders.emplace_back(libnum::num32(0.04), libnum::num32(1.44));
  MyPinball.borders.emplace_back(libnum::num32(0.02), libnum::num32(1.20));

  MyPinball.borders.emplace_back( libnum::num32(0.02), libnum::num32(1.00) );
  MyPinball.borders.emplace_back( libnum::num32(0.26), libnum::num32(0.85) );
  MyPinball.borders.emplace_back( libnum::num32(0.26), libnum::num32(0.70) );
  MyPinball.borders.emplace_back( libnum::num32(0.42), libnum::num32(0.62) );

  MyPinball.borders.emplace_back( libnum::num32(0.44), libnum::num32(0.60) );
  MyPinball.borders.emplace_back( libnum::num32(0.44), libnum::num32(0.50) );
  MyPinball.borders.emplace_back( libnum::num32(1.02), libnum::num32(0.40) );


//  MyPinball.borders.emplace_back( libnum::num32(1.02), libnum::num32(0.40) );
  MyPinball.borders.emplace_back( libnum::num32(1.26), libnum::num32(0.25) );
  MyPinball.borders.emplace_back( libnum::num32(1.26), libnum::num32(0.10) );
  MyPinball.borders.emplace_back( libnum::num32(1.42), libnum::num32(0.02) );
  MyPinball.borders.emplace_back( libnum::num32(1.58), libnum::num32(0.02) );
  MyPinball.borders.emplace_back( libnum::num32(1.74), libnum::num32(0.10) );


  MyPinball.has_locker = true;
  MyPinball.locker.clear();
  MyPinball.locker.emplace_back( libnum::num32(1.90), libnum::num32(1.46) );
  MyPinball.locker.emplace_back( libnum::num32(2.09), libnum::num32(1.20) );
  MyPinball.locker.emplace_back( libnum::num32(2.09), libnum::num32(1.10) );
  MyPinball.locker.emplace_back( libnum::num32(2.02), libnum::num32(1.10) );
  MyPinball.locker.emplace_back( libnum::num32(1.98), libnum::num32(1.10) );
  MyPinball.locker.emplace_back( libnum::num32(1.98), libnum::num32(1.20) );
  MyPinball.locker.emplace_back( libnum::num32(1.96), libnum::num32(1.34) );
  MyPinball.locker_is_enabled = false;


  MyPinball.Lock_radius = libnum::num(0.03);
  MyPinball.Lock_pos = Vector2D(libnum::num32(1.50), libnum::num32(1.64));
  MyPinball.Lock_color = RGB565_LEMONYELLOW;


  MyPinball.BucketMin = Vector2D( libnum::num32(1.42), libnum::num32(0.02) );
  MyPinball.BucketMax = Vector2D( libnum::num32(1.58), libnum::num32(0.10) );
  

  MyPinball.balls.clear();
  libnum::num32 ball_radius = libnum::num32(0.03);
  libnum::num32 ball_mass = libnum::num32(PI) * ball_radius * ball_radius;

  Vector2D ball_pos1(libnum::num32(2.075), libnum::num32(0.85));
//  Vector2D ball_vel1(libnum::num32(0.0), libnum::num32(3.5));
  Vector2D ball_vel1(libnum::num32(0.0), libnum::num32(0.0));

  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos1, ball_vel1,
                                 0.2, RGB565_LEMONYELLOW));

  MyPinball.CurrentBallToLaunch = 0;
  MyPinball.InitialPosition = ball_pos1.Clone();

/*
  Vector2D ball_pos2(libnum::num32(1.075), libnum::num32(0.45));
  Vector2D ball_vel2(libnum::num32(0.0), libnum::num32(3.5));
  MyPinball.balls.push_back(Ball(ball_radius, ball_mass, ball_pos2, ball_vel2,
                                 0.2, RGB565_LEMONYELLOW));
*/

  MyPinball.obstacles.clear();
  MyPinball.obstacles.reserve(16);

  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(1.50), libnum::num32(1.45)), 1.5, RGB565_RED, 100 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(1.74), libnum::num32(1.20)), 1.5, RGB565_RED, 100 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(1.26), libnum::num32(1.20)), 1.5, RGB565_RED, 100 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(1.50), libnum::num32(0.95)), 1.5, RGB565_RED, 100 );

  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.50), libnum::num32(1.55)), 1.5, RGB565_APPLEGREEN, 250 );
  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.70), libnum::num32(1.40)), 1.5, RGB565_APPLEGREEN, 250 );
  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.30), libnum::num32(1.40)), 1.5, RGB565_APPLEGREEN, 250 );
  MyPinball.obstacles.emplace_back( 0.06, Vector2D(libnum::num32(0.50), libnum::num32(1.25)), 1.5, RGB565_APPLEGREEN, 250 );


  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(1.15), libnum::num32(0.80)), 1.5, RGB565_LEMONYELLOW, 25 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(1.85), libnum::num32(0.80)), 1.5, RGB565_LEMONYELLOW, 25 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(1.30), libnum::num32(0.60)), 1.5, RGB565_GREEN, 10 );
  MyPinball.obstacles.emplace_back( 0.04, Vector2D(libnum::num32(1.70), libnum::num32(0.60)), 1.5, RGB565_GREEN, 10 );

  for (int i = 0; i < MyPinball.islands.size(); i++)
    MyPinball.islands[i].clear();
  MyPinball.islands.clear();



  std::vector<Vector2D> bigisland;
    bigisland.emplace_back( libnum::num32(1.02), libnum::num32(1.05) );
    bigisland.emplace_back( libnum::num32(1.02), libnum::num32(0.50) );
    bigisland.emplace_back( libnum::num32(0.56), libnum::num32(0.56) );
    bigisland.emplace_back( libnum::num32(0.58), libnum::num32(0.62) );
    bigisland.emplace_back( libnum::num32(0.74), libnum::num32(0.70) );
    bigisland.emplace_back( libnum::num32(0.74), libnum::num32(0.85) );
    bigisland.emplace_back( libnum::num32(0.98), libnum::num32(1.00) ); 
    bigisland.emplace_back( libnum::num32(0.98), libnum::num32(1.05) ); 
    MyPinball.islands.push_back(bigisland);
  
  
  std::vector<Vector2D> island1;
    island1.emplace_back( libnum::num32(1.50), libnum::num32(1.13) );
    island1.emplace_back( libnum::num32(1.43), libnum::num32(1.20) );
    island1.emplace_back( libnum::num32(1.50), libnum::num32(1.27) );
    island1.emplace_back( libnum::num32(1.57), libnum::num32(1.20) );
    MyPinball.islands.push_back(island1);

    std::vector<Vector2D> island2;
    island2.emplace_back( libnum::num32(1.49), libnum::num32(0.61) );
    island2.emplace_back( libnum::num32(1.40), libnum::num32(0.80) );
    island2.emplace_back( libnum::num32(1.50), libnum::num32(0.82) );
    island2.emplace_back( libnum::num32(1.60), libnum::num32(0.80) );
    island2.emplace_back( libnum::num32(1.51), libnum::num32(0.61) );
    MyPinball.islands.push_back(island2);

    std::vector<Vector2D> island3;
    island3.emplace_back( libnum::num32(1.12), libnum::num32(0.45) );
    island3.emplace_back( libnum::num32(1.12), libnum::num32(0.65) );
    island3.emplace_back( libnum::num32(1.14), libnum::num32(0.65) );
    island3.emplace_back( libnum::num32(1.26), libnum::num32(0.39) );
    island3.emplace_back( libnum::num32(1.26), libnum::num32(0.37) );
    MyPinball.islands.push_back(island3);

    std::vector<Vector2D> island4;
    island4.emplace_back( libnum::num32(1.88), libnum::num32(0.65) );
    island4.emplace_back( libnum::num32(1.88), libnum::num32(0.45) );
    island4.emplace_back( libnum::num32(1.74), libnum::num32(0.37) );
    island4.emplace_back( libnum::num32(1.74), libnum::num32(0.39) );
    island4.emplace_back( libnum::num32(1.86), libnum::num32(0.65) );
    MyPinball.islands.push_back(island4);

    std::vector<Vector2D> island5;
    island5.emplace_back( libnum::num32(0.12), libnum::num32(1.06) );
    island5.emplace_back( libnum::num32(0.12), libnum::num32(1.16) );
    island5.emplace_back( libnum::num32(0.19), libnum::num32(1.16) );
    island5.emplace_back( libnum::num32(0.28), libnum::num32(0.95) );
    island5.emplace_back( libnum::num32(0.28), libnum::num32(0.93) );
    MyPinball.islands.push_back(island5);

    std::vector<Vector2D> island6;
    island6.emplace_back( libnum::num32(0.88), libnum::num32(1.16) );
    island6.emplace_back( libnum::num32(0.88), libnum::num32(1.06) );
    island6.emplace_back( libnum::num32(0.72), libnum::num32(0.93) );
    island6.emplace_back( libnum::num32(0.72), libnum::num32(0.95) );
    island6.emplace_back( libnum::num32(0.81), libnum::num32(1.16) );
    MyPinball.islands.push_back(island6);


    MyPinball.flippers.clear();
    libnum::num32 flip_radius = libnum::num32(0.03);
    libnum::num32 flip_length = libnum::num32(0.2);
    libnum::num32 flip_length_short = libnum::num32(0.12);
    libnum::num32 flip_maxRotation = libnum::num32(1.0);
    libnum::num32 flip_restAngle = libnum::num32(0.5);
    libnum::num32 flip_angularVelocity = libnum::num32(10.0);
    libnum::num32 flip_restitution = libnum::num32(0.0);

    Vector2D flip_pos1( libnum::num32(1.26), libnum::num32(0.22) );
    MyPinball.flippers.emplace_back( flip_radius, flip_pos1, flip_length, -flip_restAngle, flip_maxRotation, flip_angularVelocity, flip_restitution, LEFT, RGB565_OCEANBLUE );

    Vector2D flip_pos2(libnum::num32(1.74), libnum::num32(0.22));
    MyPinball.flippers.emplace_back( flip_radius, flip_pos2, flip_length, libnum::num32(PI) + flip_restAngle, -flip_maxRotation, flip_angularVelocity, flip_restitution, RIGHT, RGB565_OCEANBLUE );


    Vector2D flip_pos3(libnum::num32(0.26), libnum::num32(0.81));
    MyPinball.flippers.emplace_back( flip_radius, flip_pos3, flip_length, -flip_restAngle, flip_maxRotation, flip_angularVelocity, flip_restitution, LEFT, RGB565_OCEANBLUE );

    Vector2D flip_pos4(libnum::num32(0.74), libnum::num32(0.81));
    MyPinball.flippers.emplace_back( flip_radius, flip_pos4, flip_length, libnum::num32(PI) + flip_restAngle, -flip_maxRotation, flip_angularVelocity, flip_restitution, RIGHT, RGB565_OCEANBLUE );

}

#endif